From 0c50d907d1321fe374ffdba3b1549114ecff2d9a Mon Sep 17 00:00:00 2001
From: Sergey Fedorov <vital.had@gmail.com>
Date: Fri, 12 Jan 2024 14:46:26 +0800
Subject: [PATCH] platform.h: add cpuRelax for PowerPC and ARM

---
 dispenso/platform.h | 16 +++++++++++++++-
 1 file changed, 15 insertions(+), 1 deletion(-)

diff --git dispenso/platform.h dispenso/platform.h
index 9aa1ca3..728265a 100644
--- dispenso/platform.h
+++ dispenso/platform.h
@@ -163,8 +163,22 @@ inline constexpr uintptr_t alignToCacheLine(uintptr_t val) {
 inline void cpuRelax() {
   asm volatile("pause" ::: "memory");
 }
+#elif defined __arm64__ || defined __aarch64__
+inline void cpuRelax() {
+  asm volatile("yield" ::: "memory");
+}
+#elif defined __powerpc__ || defined __POWERPC__
+  #if defined __APPLE__
+  inline void cpuRelax() {
+    asm volatile("or r27,r27,r27" ::: "memory");
+  }
+  #else
+  inline void cpuRelax() {
+    asm volatile("or 27,27,27" ::: "memory");
+  }
+  #endif
 #else
-// TODO: provide reasonable relax on non-x86
+// TODO: provide reasonable relax on other archs.
 inline void cpuRelax() {}
 #endif // x86-arch
 
